use utils::combination;
use utils::math::binomial;

pub fn process_part1(input: &str, litres: u32) -> u32 {
    let containers = input
        .lines()
        .map(|container| container.parse::<u32>().unwrap())
        .collect::<Vec<u32>>();
    let mut correct_combinations = Vec::new();
    for k in 1..=containers.len() {
        let num_combinations =
            binomial(containers.len().try_into().unwrap(), k.try_into().unwrap());
        for i in 1..=num_combinations {
            let res = combination::nth_lex(containers.clone(), k, i.try_into().unwrap()).unwrap();
            if res.iter().sum::<u32>() == litres {
                correct_combinations.push(res);
            }
        }
    }
    correct_combinations.len() as u32
}

pub fn process_part2(input: &str, litres: u32) -> u32 {
    let containers = input
        .lines()
        .map(|container| container.parse::<u32>().unwrap())
        .collect::<Vec<u32>>();
    let mut correct_combinations = Vec::new();
    for k in 1..=containers.len() {
        let num_combinations =
            binomial(containers.len().try_into().unwrap(), k.try_into().unwrap());
        for i in 1..=num_combinations {
            let res = combination::nth_lex(containers.clone(), k, i.try_into().unwrap()).unwrap();
            if res.iter().sum::<u32>() == litres {
                correct_combinations.push(res);
            }
        }
        if !correct_combinations.is_empty() {
            break;
        }
    }
    correct_combinations.len() as u32
}

#[cfg(test)]
mod tests {
    use super::*;

    const INPUT: &str = "20
15
10
5
5";

    #[test]
    fn part1() {
        let result = process_part1(INPUT, 25);
        assert_eq!(result, 4);
    }

    #[test]
    fn part2() {
        let result = process_part2(INPUT, 25);
        assert_eq!(result, 3);
    }
}
