use utils::math::get_divisors;

pub fn process_part1(input: u32) -> u32 {
    // slow
    //let mut house = 1;
    //loop {
    //    let mut presents = 0;
    //    for elf in 1..=house {
    //        if house % elf == 0 {
    //            presents += elf * 10;
    //            println!("{house} {presents}");
    //        }
    //    }
    //    if presents >= input {
    //        break;
    //    }
    //    house += 1;
    //}
    //house
    let mut house = 0;
    loop {
        house += 1;
        let mut divisors = get_divisors(house);
        divisors.push(house);
        if divisors.iter().sum::<u64>() * 10 >= input.into() {
            break;
        }
    }
    house as u32
}

pub fn process_part2(input: u32) -> u32 {
    let mut house = 0;
    loop {
        house += 1;
        let mut divisors = get_divisors(house);
        divisors.push(house);
        let sum = divisors.iter().filter(|&&d| house / d <= 50).sum::<u64>();
        if sum * 11 >= input.into() {
            break;
        }
    }
    house as u32
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn part1() {
        let result = process_part1(100);
        assert_eq!(result, 6);
    }
}
