use core::panic;

pub fn process_part1(input: &str) -> u32 {
    let mut boss = Character::default();
    input.lines().for_each(|line| {
        let (attribute, value) = line.split_once(": ").unwrap();
        if attribute == "Hit Points" {
            boss.hp = value.parse::<u32>().unwrap();
        } else if attribute == "Damage" {
            boss.damage = value.parse::<u32>().unwrap();
        }
    });
    let player = Character {
        hp: 50,
        mana: 500,
        ..Default::default()
    };
    // Initial state, nothing happened yet
    let start = RoundNode {
        player,
        boss,
        spent_mana: 0,
        state: State::Playing,
    };
    let mut smallest = u32::MAX;
    let mut leafs = vec![start];
    println!("start");
    // Playing, Win, Loss
    let mut stats = (0, 0, 0);
    loop {
        let current_leafs = leafs.clone();
        leafs.clear();
        // create potential rounds
        for leaf in current_leafs {
            if leaf.state == State::Playing {
                for spell in Spell::get_all() {
                    match spell.name {
                        SpellID::MagicMissile | SpellID::Drain => {
                            leafs.push(leaf.use_spell(spell, false));
                        }
                        SpellID::Shield => {
                            if !leaf
                                .player
                                .status_effects
                                .iter()
                                .any(|effect| effect.name == SpellID::Shield && effect.duration > 1)
                            {
                                leafs.push(leaf.use_spell(spell, false));
                            }
                        }
                        SpellID::Poison => {
                            if !leaf
                                .boss
                                .status_effects
                                .iter()
                                .any(|effect| effect.name == SpellID::Poison && effect.duration > 1)
                            {
                                leafs.push(leaf.use_spell(spell, false));
                            }
                        }
                        SpellID::Recharge => {
                            if !leaf.player.status_effects.iter().any(|effect| {
                                effect.name == SpellID::Recharge && effect.duration > 1
                            }) {
                                leafs.push(leaf.use_spell(spell, false));
                            }
                        }
                    }
                }
            }
            //leafs.iter().for_each(|leaf| println!("{:?}", leaf));
        }
        println!("leaf count: {}", leafs.len());
        stats.0 = 0;
        for leaf in &leafs {
            if leaf.state == State::Win && leaf.spent_mana < smallest {
                smallest = leaf.spent_mana;
            }
            match leaf.state {
                State::Playing => stats.0 += 1,
                State::Win => stats.1 += 1,
                State::Loss => stats.2 += 1,
            };
            //println!(
            //    "player hp: {}, boss hp: {}, spent mana so far: {}",
            //    leaf.player.hp, leaf.boss.hp, leaf.spent_mana
            //);
        }
        println!("Playing, win, loss: {stats:?}");
        leafs.retain(|leaf| leaf.state == State::Playing && leaf.spent_mana < smallest);
        if leafs.is_empty() {
            break;
        }
        println!(" smallest: {smallest}");
    }
    smallest
}

pub fn process_part2(input: &str) -> u32 {
    let mut boss = Character::default();
    input.lines().for_each(|line| {
        let (attribute, value) = line.split_once(": ").unwrap();
        if attribute == "Hit Points" {
            boss.hp = value.parse::<u32>().unwrap();
        } else if attribute == "Damage" {
            boss.damage = value.parse::<u32>().unwrap();
        }
    });
    let player = Character {
        hp: 50,
        mana: 500,
        ..Default::default()
    };
    // Initial state, nothing happened yet
    let start = RoundNode {
        player,
        boss,
        spent_mana: 0,
        state: State::Playing,
    };
    let mut smallest = u32::MAX;
    let mut leafs = vec![start];
    println!("start");
    // Playing, Win, Loss
    let mut stats = (0, 0, 0);
    loop {
        let current_leafs = leafs.clone();
        leafs.clear();
        // create potential rounds
        for leaf in current_leafs {
            if leaf.state == State::Playing {
                for spell in Spell::get_all() {
                    match spell.name {
                        SpellID::MagicMissile | SpellID::Drain => {
                            leafs.push(leaf.use_spell(spell, true));
                        }
                        SpellID::Shield => {
                            if !leaf
                                .player
                                .status_effects
                                .iter()
                                .any(|effect| effect.name == SpellID::Shield && effect.duration > 1)
                            {
                                leafs.push(leaf.use_spell(spell, true));
                            }
                        }
                        SpellID::Poison => {
                            if !leaf
                                .boss
                                .status_effects
                                .iter()
                                .any(|effect| effect.name == SpellID::Poison && effect.duration > 1)
                            {
                                leafs.push(leaf.use_spell(spell, true));
                            }
                        }
                        SpellID::Recharge => {
                            if !leaf.player.status_effects.iter().any(|effect| {
                                effect.name == SpellID::Recharge && effect.duration > 1
                            }) {
                                leafs.push(leaf.use_spell(spell, true));
                            }
                        }
                    }
                }
            }
            //leafs.iter().for_each(|leaf| println!("{:?}", leaf));
        }
        println!("leaf count: {}", leafs.len());
        stats.0 = 0;
        for leaf in &leafs {
            if leaf.state == State::Win && leaf.spent_mana < smallest {
                smallest = leaf.spent_mana;
            }
            match leaf.state {
                State::Playing => stats.0 += 1,
                State::Win => stats.1 += 1,
                State::Loss => stats.2 += 1,
            };
            //println!(
            //    "player hp: {}, boss hp: {}, spent mana so far: {}",
            //    leaf.player.hp, leaf.boss.hp, leaf.spent_mana
            //);
        }
        println!("Playing, win, loss: {stats:?}");
        leafs.retain(|leaf| leaf.state == State::Playing && leaf.spent_mana < smallest);
        if leafs.is_empty() {
            break;
        }
        println!(" smallest: {smallest}");
    }
    smallest
}

#[derive(Debug, Clone, PartialEq, Eq)]
enum State {
    Playing,
    Win,
    Loss,
}

#[derive(Debug, Clone)]
struct RoundNode {
    player: Character,
    boss: Character,
    spent_mana: u32,
    state: State,
}

impl RoundNode {
    fn use_spell(&self, spell: Spell, hard_mode: bool) -> RoundNode {
        if self.state != State::Playing {
            eprintln!("State: {:#?}", self.state);
            eprintln!("spell: {spell:#?}");
            panic!("State should be Playing");
        }
        let mut player = self.player.clone();
        let mut boss = self.boss.clone();
        let mut recharge_actif = false;
        if hard_mode {
            player.hp -= 1;
        }
        if player.hp == 0 {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Loss,
            };
        }
        // Player turn
        for effect in player.status_effects.iter_mut() {
            if effect.name == SpellID::Recharge {
                player.mana += effect.mana;
                if effect.duration > 1 {
                    recharge_actif = true;
                }
            }
            effect.duration -= 1;
            if effect.duration > 0 && effect.name == SpellID::Shield {
                player.armor = effect.armor;
            } else if effect.name == SpellID::Shield {
                player.armor = 0;
            }
        }
        for effect in boss.status_effects.iter_mut() {
            boss.hp = boss.hp.saturating_sub(effect.damage);
            effect.duration -= 1;
        }
        boss.status_effects = boss
            .status_effects
            .into_iter()
            .filter(|&effect| effect.duration > 0)
            .collect::<Vec<Spell>>();
        if boss.hp == 0 {
            return RoundNode {
                player,
                boss,
                spent_mana: self.spent_mana,
                state: State::Win,
            };
        }
        player.status_effects = player
            .status_effects
            .into_iter()
            .filter(|&effect| effect.duration > 0)
            .collect::<Vec<Spell>>();
        let mut dmg = 0;
        if player.mana < spell.cost {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Loss,
            };
        }
        match spell.name {
            SpellID::MagicMissile => {
                player.mana = player.mana.saturating_sub(spell.cost);
                dmg = spell.damage;
            }
            SpellID::Drain => {
                player.mana = player.mana.saturating_sub(spell.cost);
                dmg = spell.damage;
                player.hp += spell.hp;
            }
            SpellID::Shield => {
                player.mana = player.mana.saturating_sub(spell.cost);
                player.status_effects.push(spell);
                player.armor = spell.armor;
            }
            SpellID::Poison => {
                player.mana = player.mana.saturating_sub(spell.cost);
                boss.status_effects.push(spell);
            }
            SpellID::Recharge => {
                player.mana = player.mana.saturating_sub(spell.cost);
                player.status_effects.push(spell);
            }
        }
        boss.hp = boss.hp.saturating_sub(dmg);
        if boss.hp == 0 {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Win,
            };
        }
        if player.mana == 0 && !recharge_actif {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Loss,
            };
        }

        // Boss turn
        for effect in player.status_effects.iter_mut() {
            if effect.name == SpellID::Recharge {
                player.mana += effect.mana;
            }
            effect.duration -= 1;
            if effect.duration > 0 && effect.name == SpellID::Shield {
                player.armor = effect.armor;
            } else if effect.name == SpellID::Shield {
                player.armor = 0;
            }
        }
        player.status_effects = player
            .status_effects
            .into_iter()
            .filter(|&effect| effect.duration > 0)
            .collect::<Vec<Spell>>();
        for effect in boss.status_effects.iter_mut() {
            boss.hp = boss.hp.saturating_sub(effect.damage);
            effect.duration -= 1;
        }
        if boss.hp == 0 {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Win,
            };
        }
        boss.status_effects = boss
            .status_effects
            .into_iter()
            .filter(|&effect| effect.duration > 0)
            .collect::<Vec<Spell>>();
        let dmg = boss.hit_damage(&player);
        player.hp = player.hp.saturating_sub(dmg);

        if player.hp == 0 {
            return RoundNode {
                player,
                boss,
                spent_mana: spell.cost + self.spent_mana,
                state: State::Loss,
            };
        }
        RoundNode {
            player,
            boss,
            spent_mana: spell.cost + self.spent_mana,
            state: State::Playing,
        }
    }

    //fn get_leafs(&self) -> Vec<RoundNode> {
    //    let mut leafs = Vec::new();
    //    match self.state {
    //        State::Playing => {
    //            if self.next_rounds.is_empty() {
    //                return vec![self.clone()];
    //            }
    //            let mut round_leafs = Vec::new();
    //            for round in self.next_rounds.clone() {
    //                round_leafs = round.get_leafs();
    //            }
    //            leafs.append(&mut round_leafs);
    //            leafs
    //        }
    //        State::Win => vec![self.clone()],
    //        State::Loss => vec![self.clone()],
    //    }
    //}
}

#[derive(Debug, Clone, Copy, PartialEq, Eq)]
enum SpellID {
    MagicMissile,
    Drain,
    Shield,
    Poison,
    Recharge,
}

#[derive(Debug, Clone, Copy)]
struct Spell {
    name: SpellID,
    hp: u32,
    damage: u32,
    mana: u32,
    armor: u32,
    duration: u32,
    cost: u32,
}

impl Spell {
    fn get_all() -> Vec<Spell> {
        vec![
            Spell {
                name: SpellID::MagicMissile,
                hp: 0,
                damage: 4,
                mana: 0,
                armor: 0,
                duration: 0,
                cost: 53,
            },
            Spell {
                name: SpellID::Drain,
                hp: 2,
                damage: 2,
                mana: 0,
                armor: 0,
                duration: 0,
                cost: 73,
            },
            Spell {
                name: SpellID::Shield,
                hp: 0,
                damage: 0,
                mana: 0,
                armor: 7,
                duration: 6,
                cost: 113,
            },
            Spell {
                name: SpellID::Poison,
                hp: 0,
                damage: 3,
                mana: 0,
                armor: 0,
                duration: 6,
                cost: 173,
            },
            Spell {
                name: SpellID::Recharge,
                hp: 0,
                damage: 0,
                mana: 101,
                armor: 0,
                duration: 5,
                cost: 229,
            },
        ]
    }
}

#[derive(Debug, Default, Clone)]
struct Character {
    hp: u32,
    damage: u32,
    armor: u32,
    mana: u32,
    status_effects: Vec<Spell>,
}

impl Character {
    fn hit_damage(&self, other: &Self) -> u32 {
        let damage = self.damage.saturating_sub(other.armor);
        if damage > 0 {
            damage
        } else {
            1
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn part1_1() {
        let boss = Character {
            hp: 13,
            damage: 8,
            ..Default::default()
        };
        let player = Character {
            hp: 10,
            mana: 250,
            ..Default::default()
        };
        // Initial state, nothing happened yet
        let start = RoundNode {
            player,
            boss,
            spent_mana: 0,
            state: State::Playing,
        };
        let mut smallest = u32::MAX;
        let mut leafs = vec![start];
        println!("start");
        loop {
            let current_leafs = leafs.clone();
            leafs.clear();
            // create potential rounds
            for leaf in current_leafs {
                if leaf.state == State::Playing {
                    for spell in Spell::get_all() {
                        leafs.push(leaf.use_spell(spell, false));
                    }
                }
            }
            let mut all_done = true;
            for leaf in &leafs {
                if leaf.state == State::Win && leaf.spent_mana < smallest {
                    smallest = leaf.spent_mana;
                }
                if leaf.state == State::Playing {
                    all_done = false;
                }
                //println!(
                //    "player hp: {}, boss hp: {}, spent mana so far: {}",
                //    leaf.player.hp, leaf.boss.hp, leaf.spent_mana
                //);
            }
            if all_done {
                break;
            }
            println!(" smallest: {smallest}");
        }
        assert_eq!(smallest, 226);
    }

    #[test]
    fn part1_2() {
        let boss = Character {
            hp: 14,
            damage: 8,
            ..Default::default()
        };
        let player = Character {
            hp: 10,
            mana: 250,
            ..Default::default()
        };
        // Initial state, nothing happened yet
        let start = RoundNode {
            player,
            boss,
            spent_mana: 0,
            state: State::Playing,
        };
        let mut smallest = u32::MAX;
        let mut leafs = vec![start];
        println!("start");
        loop {
            let current_leafs = leafs.clone();
            leafs.clear();
            // create potential rounds
            for leaf in current_leafs {
                if leaf.state == State::Playing {
                    for spell in Spell::get_all() {
                        leafs.push(leaf.use_spell(spell, false));
                    }
                }
            }
            let mut all_done = true;
            for leaf in &leafs {
                if leaf.state == State::Win && leaf.spent_mana < smallest {
                    smallest = leaf.spent_mana;
                }
                if leaf.state == State::Playing {
                    all_done = false;
                }
                //println!(
                //    "player hp: {}, boss hp: {}, spent mana so far: {}",
                //    leaf.player.hp, leaf.boss.hp, leaf.spent_mana
                //);
            }
            if all_done {
                break;
            }
            println!(" smallest: {smallest}");
        }
        assert_eq!(smallest, 641);
    }

    #[allow(dead_code)]
    fn init_characters() -> (Character, Character) {
        let player = Character {
            hp: 50,
            mana: 500,
            ..Default::default()
        };
        let boss = Character {
            hp: 14,
            damage: 8,
            ..Default::default()
        };
        (player, boss)
    }

    #[test]
    fn part2() {
        let result = process_part2("");
        assert_eq!(result, 0);
    }
}
